/*
 * $Id: main.h 1042 2009-06-05 02:58:33Z corner-face-jacks $
 *
 * Fri 05 Jun 2009 02:58:33 IST 
 *
 * Core GTK-based user interface (beta)
 *
 * Copyright (C) 2009 Paul Mc Quade  <corner.face.jacks@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * -----------------------------------------------------------------------------
 * First run lxdream.glade through gtk-builder-convert with this command:
 * gtk-builder-convert lxdream.glade lxdream.xml
 *
 * Compile using:
 * gcc -Wall -g -o lxdream main.c `pkg-config --cflags --libs gtk+-2.0` -export-dynamic
 *
 * -----------------------------------------------------------------------------
 *      
 *  Then execute it using:
  ./lxdream
*/
#include <gtk/gtk.h>

/* store the widgets which may need to be accessed in a typedef struct */
typedef struct
{
	GtkBuilder      *builder; 
        GtkWidget       *window;
        GtkWidget       *statusbar;
        GtkWidget	*drawingarea;
        guint           statusbar_context_id;
        gchar           *filename;
} lxdream;


/* location of UI XML file relative to path in which program is running */
#define BUILDER_XML_FILE "lxdream.xml"

/* window callback prototypes */
void on_window_destroy (GtkObject *object, gpointer user_data);

/* File menu callbacks prototypes */
void on_quit_menu_item_activate (GtkObject *object, gpointer user_data);

/* Options menu callbacks prototypes */
void on_about_menu_item_activate(GtkMenuItem *menuitem, lxdream *emulator);

/* misc. function prototypes */
void error_message (const gchar *message);
gboolean init_app (lxdream *emulator);
void reset_default_status (lxdream *emulator);


int
main (int argc, char *argv[])
{
	lxdream  *emulator;
	
	/* allocate the memory needed by our lxdream struct */
	emulator = g_slice_new (lxdream); 
  
	/* initialize GTK+ libraries */
	gtk_init (&argc, &argv);

	if (init_app (emulator) == FALSE) return 1; /* error loading UI */

	/* show the window */   
	gtk_widget_show (emulator->window);

	/* enter GTK+ main loop */
	gtk_main ();

	/* free memory we allocated for lxdream struct */
	g_slice_free (lxdream, emulator);
     
    return 0;
}


void 
on_window_destroy (GtkObject *object, gpointer user_data)
{
    gtk_main_quit ();
}

void
on_quit_menu_item_activate (GtkObject *object, gpointer user_data)
{
    gtk_main_quit ();
}

void
on_about_menu_item_activate(GtkMenuItem *menuitem, lxdream *emulator)
{
//
    gtk_main_quit ();

//
/*
gtk_show_about_dialog (GTK_WINDOW (emulator->window)//,
			       "authors", authors,
			       "comments", comments,
			       "copyright", copyright,
			       "version", "1.0",
			       "website", "http://www.lxdream.org",
			       "program-name", "lxdream",
			       "logo-icon-name", GTK_STOCK_EDIT,
			       NULL//);
*/
}


/*
We call init_app() when our program is starting to load our lxdream struct
with references to the widgets we need. This is done using GtkBuilder to read
the XML file we created using Glade.
*/
gboolean 
init_app (lxdream *emulator)
{
        GtkBuilder              *builder;
        GError                  *err=NULL;      
        guint                   id;
        
        /* use GtkBuilder to build our interface from the XML file */
        builder = gtk_builder_new ();
        if (gtk_builder_add_from_file (builder, BUILDER_XML_FILE, &err) == 0)
        {
                error_message (err->message);
                g_error_free (err);
                return FALSE;
        }
        
        /* get the widgets which will be referenced in callbacks */
        emulator->window = GTK_WIDGET (gtk_builder_get_object (builder, 
                                                             "window"));
        emulator->statusbar = GTK_WIDGET (gtk_builder_get_object (builder, 
                                                                "statusbar"));
       
        /* connect signals, passing our lxdream struct as user data */
        gtk_builder_connect_signals (builder, emulator);
                    
        /* free memory used by GtkBuilder object */
        g_object_unref (G_OBJECT (builder));
               
        /* set the default icon to the GTK "edit" icon */
        //gtk_window_set_default_icon_name (GTK_STOCK_EDIT);
	gtk_window_set_default_icon_name (GTK_STOCK_QUIT);
        //gtk_window_set_default_icon_name ("lxdream.png");

        /* setup and initialize our statusbar */
        id = gtk_statusbar_get_context_id (GTK_STATUSBAR (emulator->statusbar),
                                           "lxdream");
        emulator->statusbar_context_id = id;
        reset_default_status (emulator);
        
        /* set filename to NULL since we don't have an open/saved file yet */
        emulator->filename = NULL;
        
        return TRUE;
}

void
error_message (const gchar *message)
{
/*
        GtkWidget               *dialog;
        
        // log to terminal window //
        g_warning (message);
        
        // create an error message dialog and display modally to the user //
        dialog = gtk_message_dialog_new (NULL, 
                                         GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                                         GTK_MESSAGE_ERROR,
                                         GTK_BUTTONS_OK,
                                         message);
        
        gtk_window_set_title (GTK_WINDOW (dialog), "Error!");
        gtk_dialog_run (GTK_DIALOG (dialog));      
        gtk_widget_destroy (dialog);         
*/
}

void 
reset_default_status (lxdream *emulator)
{      
	gchar           *file;
        gchar           *status;
        
        if (emulator->filename == NULL)
        {
                file = g_strdup ("Startup");
        }
        else file = g_path_get_basename (emulator->filename);
        
        status = g_strdup_printf ("Status: %s", file);
        gtk_statusbar_pop (GTK_STATUSBAR (emulator->statusbar),
                           emulator->statusbar_context_id);
        gtk_statusbar_push (GTK_STATUSBAR (emulator->statusbar),
                            emulator->statusbar_context_id, status);
        g_free (status);
        g_free (file);
}
